package at.gv.egiz.eaaf.core.api.gui;

import java.io.InputStream;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.velocity.VelocityContext;

import at.gv.egiz.eaaf.core.exceptions.GUIBuildException;

public interface IVelocityGuiFormBuilder extends IGUIFormBuilder {

	
	/**
	 * Generate a new {@link VelocityContext} and populate it with MOA-ID GUI parameters
	 * 
	 * @param config
	 * @return
	 */
	public VelocityContext generateVelocityContextFromConfiguration(IVelocityGUIBuilderConfiguration config);
	

	/**
	 * Load the template from different resources
	 * 
	 * @param config
	 * @return An {@link InputStream} but never null. The {@link InputStream} had to be closed be the invoking method
	 * @throws GUIBuildException
	 */
	public InputStream getTemplateInputStream(IVelocityGUIBuilderConfiguration config) throws GUIBuildException;


	/**
	 * Parse a GUI template, with parameters into a http servlet-response 
	 * and use the default http-response content-type.
	 * <br><br>
	 *  The parser use the <code>VelocityEngine</code> as internal template evaluator. 
	 * 
     * @param httpReq http-request object
	 * @param httpResp http-response object
     * @param config Configuration object
	 * @param loggerName String, which should be used from logger
	 * 
	 * @throws GUIBuildException
	 */
	void build(HttpServletRequest httpReq, HttpServletResponse httpResp, IVelocityGUIBuilderConfiguration config, String loggerName)
			throws GUIBuildException;


	/**
	 * Parse a GUI template, with parameters into a http servlet-response.
	 * <br><br>
	 *  The parser use the <code>VelocityEngine</code> as internal template evaluator. 
	 * 
	 * @param httpReq http-request object
	 * @param httpResp http-response object
	 * @param config Configuration object
	 * @param contentType http-response content-type, which should be set
	 * @param loggerName String, which should be used from logger
	 * 
	 * @throws GUIBuildException
	 */
	void build(HttpServletRequest httpReq, HttpServletResponse httpResp, IVelocityGUIBuilderConfiguration config, String contentType,
			String loggerName) throws GUIBuildException;
}
