/*******************************************************************************
 * Copyright 2017 Graz University of Technology
 * EAAF-Core Components has been developed in a cooperation between EGIZ,  
 * A-SIT Plus, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *  
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
package at.gv.egiz.eaaf.core.api.idp.process;

import at.gv.egiz.eaaf.core.exceptions.EAAFException;
import at.gv.egiz.eaaf.core.exceptions.EAAFStorageException;
import at.gv.egiz.eaaf.core.impl.idp.process.ProcessInstance;
import at.gv.egiz.eaaf.core.impl.idp.process.dao.ProcessInstanceStore;

public interface ProcessInstanceStoreDAO {

	/**
	 * Stores a {@link ProcessInstance} defined by {@code pIStore} in the
	 * database.
	 * 
	 * @param pIStore
	 *            the {@link ProcessInstanceStore} to persist.
	 * @throws EAAFStorageException
	 *             is thrown if a problem occurs while accessing the database.
	 */
	void saveOrUpdate(ProcessInstanceStore pIStore) throws EAAFException;

	/**
	 * Returns a {@link ProcessInstanceStore}, defined by
	 * {@code processInstanceID} from the database, or {@code null} if the
	 * object could not be found.
	 * 
	 * @param processInstanceId
	 *            the id of the {@code ProcessInstanceStore} to retrieve.
	 * @return a ProcessInstanceStore, or {@code null}.
	 * @throws EAAFStorageException
	 *             is thrown if a problem occurs while accessing the database.
	 */
	ProcessInstanceStore load(String processInstanceId) throws EAAFException;

	/**
	 * Deletes the {@link ProcessInstance} corresponding with the
	 * {@code processInstanceId}.
	 * 
	 * @param processInstanceId
	 *            the id of the {@code ProcessInstance} to be deleted.
	 * @throws EAAFStorageException
	 *             is thrown if a problem occurs while accessing the database.
	 */
	void remove(String processInstanceId) throws EAAFException;

}
