/*******************************************************************************
 * Copyright 2017 Graz University of Technology
 * EAAF-Core Components has been developed in a cooperation between EGIZ,  
 * A-SIT Plus, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *  
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
package at.gv.egiz.eaaf.core.impl.data;

import java.io.Serializable;

import at.gv.egiz.eaaf.core.api.idp.slo.SLOInformationInterface;


/**
 * @author tlenz
 *
 */
public class SLOInformationImpl implements SLOInformationInterface, Serializable {
	
	private static final long serialVersionUID = 295577931870512387L;
	private String sessionIndex = null;
	private String nameID = null;
	private String protocolType = null;
	private String nameIDFormat = null;
	private String binding = null;
	private String serviceURL = null;
	private String authURL = null;
	private String spEntityID = null;
	
	public SLOInformationImpl(String authURL, String spEntityID, String sessionID, String nameID, String nameIDFormat, String protocolType) {
		new SLOInformationImpl(authURL, spEntityID, sessionID, nameID, nameIDFormat, protocolType, null, null);
	}
	
	public SLOInformationImpl(String authURL, String spEntityID, String sessionID, String nameID, String nameIDFormat, String protocolType, String sloBinding, String sloLocationURL) {
		this.sessionIndex = sessionID;
		this.nameID = nameID;
		this.nameIDFormat = nameIDFormat;
		this.protocolType = protocolType;
		this.spEntityID = spEntityID;
		
		if (authURL.endsWith("/"))
			this.authURL = authURL.substring(0, authURL.length()-1);
		else
			this.authURL = authURL;
		

		this.binding = sloBinding;
		this.serviceURL = sloLocationURL;
							
	}
	
	
	/**
	 * 
	 */
	public SLOInformationImpl() {
		
	}


	
	/**
	 * @return the spEntityID
	 */
	public String getSpEntityID() {
		return spEntityID;
	}

	/* (non-Javadoc)
	 * @see at.gv.egovernment.moa.id.data.SLOInformationInterface#getSessionIndex()
	 */
	@Override
	public String getSessionIndex() {
		return sessionIndex;
		
	}

	/* (non-Javadoc)
	 * @see at.gv.egovernment.moa.id.data.SLOInformationInterface#getUserNameIdentifier()
	 */
	@Override
	public String getUserNameIdentifier() {
		return nameID;
		
	}


	/**
	 * @param sessionIndex the sessionIndex to set
	 */
	public void setSessionIndex(String sessionIndex) {
		this.sessionIndex = sessionIndex;
	}


	/**
	 * @param nameID the nameID to set
	 */
	public void setUserNameIdentifier(String nameID) {
		this.nameID = nameID;
	}

	

	/**
	 * @param protocolType the protocolType to set
	 */
	public void setProtocolType(String protocolType) {
		this.protocolType = protocolType;
	}


	/* (non-Javadoc)
	 * @see at.gv.egovernment.moa.id.data.SLOInformationInterface#getProtocolType()
	 */
	@Override
	public String getProtocolType() {
		return protocolType;
	}


	/* (non-Javadoc)
	 * @see at.gv.egovernment.moa.id.data.SLOInformationInterface#getUserNameIDFormat()
	 */
	@Override
	public String getUserNameIDFormat() {
		return this.nameIDFormat;
	}


	/**
	 * @param nameIDFormat the nameIDFormat to set
	 */
	public void setNameIDFormat(String nameIDFormat) {
		this.nameIDFormat = nameIDFormat;
	}

	/**
	 * @return the binding
	 */
	public String getBinding() {
		return binding;
	}

	/**
	 * @return the serviceURL
	 */
	public String getServiceURL() {
		return serviceURL;
	}

	/**
	 * @return the authURL from requested IDP without ending /
	 */
	public String getAuthURL() {
		return authURL;
	}

	/**
	 * @param spEntityID the spEntityID to set
	 */
	public void setSpEntityID(String spEntityID) {
		this.spEntityID = spEntityID;
	}
	
	
	
	
	

}
