/*******************************************************************************
 * Copyright 2017 Graz University of Technology
 * EAAF-Core Components has been developed in a cooperation between EGIZ,  
 * A-SIT Plus, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *  
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
package at.gv.egiz.eaaf.core.impl.idp.builder.attributes;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import at.gv.egiz.eaaf.core.api.data.EAAFConstants;
import at.gv.egiz.eaaf.core.api.idp.IAttributeGenerator;
import at.gv.egiz.eaaf.core.api.idp.IAuthData;
import at.gv.egiz.eaaf.core.api.idp.IPVPAttributeBuilder;
import at.gv.egiz.eaaf.core.api.idp.ISPConfiguration;
import at.gv.egiz.eaaf.core.exceptions.AttributeBuilderException;
import at.gv.egiz.eaaf.core.exceptions.UnavailableAttributeException;

@PVPMETADATA
public class BPKAttributeBuilder implements IPVPAttributeBuilder {
	
	private static final Logger log = LoggerFactory.getLogger(BPKAttributeBuilder.class);	
	public static final String DELIMITER_BPKTYPE_BPK = ":";
	
	public String getName() {
		return BPK_NAME;
	}
	
	public <ATT> ATT build(ISPConfiguration oaParam, IAuthData authData,
			IAttributeGenerator<ATT> g) throws AttributeBuilderException {
		String result = getBpkForSP(authData);				
		log.trace("Authenticate user with bPK/wbPK: " + result);		
		return g.buildStringAttribute(BPK_FRIENDLY_NAME, BPK_NAME, result);
		
	}
	
	public <ATT> ATT buildEmpty(IAttributeGenerator<ATT> g) {
		return g.buildEmptyAttribute(BPK_FRIENDLY_NAME, BPK_NAME);
	}
	
	/**
	 * Generate the bPK String for this specific SP
	 * 
	 * @param authData
	 * @return
	 * @throws UnavailableAttributeException
	 */
	protected String getBpkForSP(IAuthData authData) throws UnavailableAttributeException {
		String bpk = attrMaxSize(authData.getBPK());
		String type = removeBpkTypePrefix(authData.getBPKType());
		
		if (StringUtils.isEmpty(bpk))
			throw new UnavailableAttributeException(BPK_NAME);
								
		return type + DELIMITER_BPKTYPE_BPK + bpk;
		
	}
	
	/**
	 * Limit the attribute value to maximum size
	 * 
	 * @param attr
	 * @return
	 */
	protected String attrMaxSize(String attr) {
		if (attr != null && attr.length() > BPK_MAX_LENGTH) {
			attr = attr.substring(0, BPK_MAX_LENGTH);
		}
		return attr;
		
	}
	
	/**
	 * Remove bPKType prefix if available
	 * 
	 * @param type
	 * @return
	 */
	protected String removeBpkTypePrefix(String type) {
		if (type.startsWith(EAAFConstants.URN_PREFIX_WBPK))
			return type.substring((EAAFConstants.URN_PREFIX_WBPK).length());
		
		else if (type.startsWith(EAAFConstants.URN_PREFIX_CDID)) 
			return type.substring((EAAFConstants.URN_PREFIX_CDID).length());
		
		else if (type.startsWith(EAAFConstants.URN_PREFIX_EIDAS)) 
			return type.substring((EAAFConstants.URN_PREFIX_EIDAS).length());
		
		else
			return type;
		
	}
}
