/*******************************************************************************
 * Copyright 2017 Graz University of Technology
 * EAAF-Core Components has been developed in a cooperation between EGIZ,  
 * A-SIT Plus, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *  
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
package at.gv.egiz.eaaf.core.impl.idp.controller.tasks;

import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import at.gv.egiz.eaaf.core.api.idp.process.ExecutionContext;
import at.gv.egiz.eaaf.core.api.idp.process.ProcessEngine;
import at.gv.egiz.eaaf.core.exceptions.EAAFException;
import at.gv.egiz.eaaf.core.exceptions.TaskExecutionException;
import at.gv.egiz.eaaf.core.impl.idp.auth.modules.AbstractAuthServletTask;
import at.gv.egiz.eaaf.core.impl.idp.auth.modules.ModuleRegistration;
import at.gv.egiz.eaaf.core.impl.idp.controller.protocols.RequestImpl;
import at.gv.egiz.eaaf.core.impl.idp.process.ExecutionContextImpl;

/**
 * @author tlenz
 *
 */
@Component("RestartAuthProzessManagement")
public class RestartAuthProzessManagement  extends AbstractAuthServletTask {
	private static final Logger log = LoggerFactory.getLogger(RestartAuthProzessManagement.class);
	
	@Autowired ProcessEngine processEngine;
	
	/* (non-Javadoc)
	 * @see at.gv.egovernment.moa.id.process.springweb.MoaIdTask#execute(at.gv.egovernment.moa.id.process.api.ExecutionContext, javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
	 */
	@Override
	public void execute(ExecutionContext executionContext, HttpServletRequest request, HttpServletResponse response)
			throws TaskExecutionException {
		try {
			
			if (this.pendingReq.isAbortedByUser()) {
				log.debug("AuthProcess was stopped. Forward to finalization ... ");
				performRedirectToProtocolFinialization(executionContext, pendingReq, request, response);
				
			} else {						
				//create a new execution context and copy all elements to new context
				final ExecutionContext newec = new ExecutionContextImpl(); 
				final Set<String> entries = executionContext.keySet();
				for (final String key : entries) {
					newec.put(key, executionContext.get(key));
					
				}
				
				log.debug("Select new auth.-process and restart restart process-engine ... ");
				
				// select and create new process instance
				final String processDefinitionId = ModuleRegistration.getInstance().selectProcess(newec);
				if (processDefinitionId == null) {
					log.warn("No suitable authentication process found for SessionID " + pendingReq.getPendingRequestId());
					throw new EAAFException("process.02", new Object[] { pendingReq.getPendingRequestId()});
				}			
				
				final String processInstanceId = processEngine.createProcessInstance(processDefinitionId, newec);
	
				// keep process instance id in moa session
				((RequestImpl)pendingReq).setProcessInstanceId(processInstanceId);
	
				// make sure pending request has been persisted before running the process
				try {
					requestStoreage.storePendingRequest(pendingReq);
					
				} catch (final EAAFException e) {
					log.error("Database Error! MOASession is not stored!");
					throw new EAAFException("internal.02", null);
					
				}
				
				log.info("Restart process-engine with auth.process:" + processDefinitionId);
				
				// start process
				processEngine.start(pendingReq);
				
			}
			
		} catch (final EAAFException e) {
			throw new TaskExecutionException(pendingReq, e.getMessage(), e);
			
		} catch (final Exception e) {
			log.warn("RestartAuthProzessManagement has an internal error", e);
			throw new TaskExecutionException(pendingReq, e.getMessage(), e);
			
		}			
		
	}

}
