/*******************************************************************************
 * Copyright 2017 Graz University of Technology
 * EAAF-Core Components has been developed in a cooperation between EGIZ,  
 * A-SIT Plus, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *  
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
package at.gv.egiz.eaaf.core.impl.idp.process;

import java.io.Serializable;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import at.gv.egiz.eaaf.core.api.idp.process.ExecutionContext;

/**
 * ExecutionContext implementation, related to a certain process instance.
 * 
 * @author tknall
 * 
 */
public class ExecutionContextImpl implements ExecutionContext {

	private static final long serialVersionUID = 1L;

	private Map<String, Serializable> ctxData = Collections.synchronizedMap(new HashMap<String, Serializable>());

	private String processInstanceId;

	/**
	 * Creates a new instance.
	 */
	public ExecutionContextImpl() {
	}

	/**
	 * Creates a new instance and associated it with a certain process instance.
	 */
	public ExecutionContextImpl(String processInstanceId) {
		this.processInstanceId = processInstanceId;
	}

	@Override
	public void setProcessInstanceId(String processInstanceId) {
		this.processInstanceId = processInstanceId;
	}

	@Override
	public String getProcessInstanceId() {
		return processInstanceId;
	}

	@Override
	public Serializable get(String key) {
		return ctxData.get(key);
	}

	@Override
	public Serializable remove(String key) {
		return ctxData.remove(key);
	}

	@Override
	public void put(String key, Serializable object) {
		ctxData.put(key, object);
	}

	@Override
	public Set<String> keySet() {
		return Collections.unmodifiableSet(ctxData.keySet());
	}

	@Override
	public String toString() {
		StringBuilder builder = new StringBuilder();
		builder.append("ExecutionContextImpl [");
		builder.append("id=").append(processInstanceId);
		builder.append(", variables=");
		builder.append(ctxData.keySet());
		builder.append("]");
		return builder.toString();
	}

}
