/*******************************************************************************
 * Copyright 2017 Graz University of Technology
 * EAAF-Core Components has been developed in a cooperation between EGIZ,  
 * A-SIT Plus, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *  
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
package at.gv.egiz.eaaf.core.impl.idp.process.dao;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import at.gv.egiz.eaaf.core.api.idp.process.ProcessInstanceStoreDAO;
import at.gv.egiz.eaaf.core.api.storage.ITransactionStorage;
import at.gv.egiz.eaaf.core.exceptions.EAAFException;

/**
 * Database backed implementation of the {@link ProcessInstanceStoreDAO}
 * interface.
 */
@Service("ProcessInstanceStoreage")
public class ProcessInstanceStoreDAOImpl implements ProcessInstanceStoreDAO {

	private Logger log = LoggerFactory.getLogger(getClass());

	@Autowired ITransactionStorage transactionStorage;
	
	@Override
	public void saveOrUpdate(ProcessInstanceStore pIStore) throws EAAFException {
		try {
			transactionStorage.put(pIStore.getProcessInstanceId(), pIStore, -1);
			log.debug("Store process instance with='{}' in the database.", pIStore.getProcessInstanceId());
			
		} catch (EAAFException e) {
			log.warn("ProcessInstanceStore could not be persisted to the database.");
			throw e;
		}
	}

	@Override
	public ProcessInstanceStore load(String processInstanceId) throws EAAFException {
		log.debug("Retrieve the ProcessInstanceStore for id='{}' from the database.", processInstanceId);
		ProcessInstanceStore result = null;
		try {
			result = transactionStorage.get(processInstanceId, ProcessInstanceStore.class);

		} catch (Exception e) {
			log.error("There are multiple persisted processes with the same process instance id '{}'",
				processInstanceId);

			throw e;
		}
			
		if (result != null) {
			log.debug("Found process instance store for instance '{}'.", processInstanceId);
			
		} else {
			log.debug("Unable to find process instance store for instance '{}'.", processInstanceId);
			
		}
		
		return result;
	}

	@Override
	public void remove(String processInstanceId) throws EAAFException {

		log.debug("Delete the ProcessInstanceStore for id='{}' from the database.", processInstanceId);		
				
		if (transactionStorage.containsKey(processInstanceId))
			transactionStorage.remove(processInstanceId);
		else 
			log.trace("ProcessInstanceStore for id='{}' was not found and could therefore not be deleted.", processInstanceId);
	}

}
