/*******************************************************************************
 * Copyright 2017 Graz University of Technology
 * EAAF-Core Components has been developed in a cooperation between EGIZ,  
 * A-SIT Plus, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *  
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
package at.gv.egiz.eaaf.core.impl.idp.process.model;

import java.io.Serializable;

import org.apache.commons.collections4.CollectionUtils;

import at.gv.egiz.eaaf.core.api.idp.process.Task;

/**
 * Represents information about a single task to be performed upon process execution.
 * @author tknall
 *
 */
public class TaskInfo extends ProcessNode implements Serializable {

	private static final long serialVersionUID = 1L;
	private static final boolean DEFAULT_ASYNC = false;
	
	private String taskImplementingClass;
	private boolean async = DEFAULT_ASYNC;
	
	/**
	 * Determines if the task is marked asynchronous ({@code true}) or synchronous ({@code false}).
	 * @return A flag indicating if the task should be executed asynchronously or synchronously. (Default: {@code false})
	 */
	public boolean isAsync() {
		return async;
	}

	/**
	 * Marks a task to executed asynchronously ({@code true}) or synchronously ({@code false}).
	 * @param async The flag.
	 */
	public void setAsync(boolean async) {
		this.async = async;
	}

	/**
	 * Returns the class that implements the actual task (must implement {@link Task}).
	 * @return The task implementing class.
	 */
	public String getTaskImplementingClass() {
		return taskImplementingClass;
	}

	/**
	 * Sets the class that implements the actual task (must implement {@link Task}).
	 * @param taskImplementingClass The task implementing class.
	 */
	public void setTaskImplementingClass(String taskImplementingClass) {
		this.taskImplementingClass = taskImplementingClass;
	}

	@Override
	public String toString() {
		StringBuilder builder = new StringBuilder();
		if (getId() != null) {
			builder.append("id=");
			builder.append(getId());
		}
		if (async != DEFAULT_ASYNC) {
			if (builder.length() > 0) {
				builder.append(", ");
			}
			builder.append("async=");
			builder.append(async);
		}
		if (taskImplementingClass != null) {
			if (builder.length() > 0) {
				builder.append(", ");
			}
			builder.append("taskImplementingClass=");
			builder.append(taskImplementingClass);
		}
		if (CollectionUtils.isNotEmpty(getIncomingTransitions())) {
			if (builder.length() > 0) {
				builder.append(", ");
			}
			builder.append("incomingTransitions=");
			builder.append(getIncomingTransitions());
		}
		if (CollectionUtils.isNotEmpty(getOutgoingTransitions())) {
			if (builder.length() > 0) {
				builder.append(", ");
			}
			builder.append("outgoingTransitions=");
			builder.append(getOutgoingTransitions());
		}
		builder.insert(0, "TaskInfo [");
		builder.append("]");
		return builder.toString();
	}

}
