/*******************************************************************************
 * Copyright 2017 Graz University of Technology
 * EAAF-Core Components has been developed in a cooperation between EGIZ,  
 * A-SIT Plus, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *  
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
package at.gv.egiz.eaaf.core.impl.idp.process.support;

import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;

/**
 * Holder for a secure random instance following the initialization on demand holder design pattern. The secure random
 * instance is a singleton that is initialized on first usage.
 * 
 * @author tknall
 * 
 */
public class SecureRandomHolder {

	private SecureRandomHolder() {
	}

	private static final SecureRandom SRND_INSTANCE;
	static {
		try {
			SRND_INSTANCE = SecureRandom.getInstance("SHA1PRNG");
		} catch (NoSuchAlgorithmException e) {
			throw new RuntimeException("Unable to instantiate SHA1PRNG.", e);
		}
	}

	/**
	 * Returns a secure random generator instance.
	 * @return The secure random instance.
	 */
	public static SecureRandom getInstance() {
		return SecureRandomHolder.SRND_INSTANCE;
	}

}