package at.gv.egiz.eaaf.core.impl.utils;

import java.security.NoSuchAlgorithmException;

import javax.annotation.PostConstruct;
import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.SSLContext;

import org.apache.http.client.config.RequestConfig;
import org.apache.http.conn.socket.LayeredConnectionSocketFactory;
import org.apache.http.conn.ssl.NoopHostnameVerifier;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import at.gv.egiz.eaaf.core.api.idp.IConfiguration;

@Service
public class HttpClientFactory {
	private static final Logger log = LoggerFactory.getLogger(HttpClientFactory.class);	
	@Autowired(required=true) private IConfiguration basicConfig;
	
	public static final String PROP_CONFIG_CLIENT_HTTP_CONNECTION_POOL_USE = "client.http.connection.pool.use";
	public static final String PROP_CONFIG_CLIENT_HTTP_CONNECTION_POOL_MAXTOTAL = "client.http.connection.pool.maxtotal";
	public static final String PROP_CONFIG_CLIENT_HTTP_CONNECTION_POOL_MAXPERROUTE = "client.http.connection.pool.maxperroute";
	public static final String PROP_CONFIG_CLIENT_HTTP_CONNECTION_TIMEOUT_SOCKET = "client.http.connection.timeout.socket";
	public static final String PROP_CONFIG_CLIENT_HTTP_CONNECTION_TIMEOUT_CONNECTION = "client.http.connection.timeout.connection";
	public static final String PROP_CONFIG_CLIENT_HTTP_CONNECTION_TIMEOUT_REQUEST = "client.http.connection.timeout.request";
	public static final String PROP_CONFIG_CLIENT_HTTP_SSL_HOSTNAMEVERIFIER_TRUSTALL = "client.http.ssl.hostnameverifier.trustall";
	
	// default configuration values
	public static final String DEFAULT_CONFIG_CLIENT_HTTP_CONNECTION_TIMEOUT_SOCKET = "300";
	public static final String DEFAULT_CONFIG_CLIENT_HTTP_CONNECTION_TIMEOUT_CONNECTION = "300";
	public static final String DEFAULT_CONFIG_CLIENT_HTTP_CONNECTION_TIMEOUT_REQUEST = "1500";
	public static final String DEFAULT_CONFIG_CLIENT_HTTP_CONNECTION_POOL_MAXTOTAL = "500";
	public static final String DEFAULT_CONFIG_CLIENT_HTTP_CONNECTION_POOL_MAXPERROUTE = "100";
	
	private HttpClientBuilder httpClientBuilder = null;
	
	/**
	 * Return an instance of a Apache HTTP client
	 * 
	 * @return
	 */
	public CloseableHttpClient getHttpClient() {
		return httpClientBuilder.build();
		
	}
	
	
	@PostConstruct
	private void initalize() {
		//initialize http client 
		log.trace("Initializing HTTP Client-builder ... ");
		httpClientBuilder = HttpClients.custom();
		 
		//set default request configuration
		final RequestConfig requestConfig = RequestConfig.custom()
											.setConnectTimeout(Integer.valueOf(basicConfig.getBasicConfiguration(
													PROP_CONFIG_CLIENT_HTTP_CONNECTION_TIMEOUT_CONNECTION, 
													DEFAULT_CONFIG_CLIENT_HTTP_CONNECTION_TIMEOUT_CONNECTION)) * 1000)
											.setConnectionRequestTimeout(Integer.valueOf(basicConfig.getBasicConfiguration(
													PROP_CONFIG_CLIENT_HTTP_CONNECTION_TIMEOUT_REQUEST, 
													DEFAULT_CONFIG_CLIENT_HTTP_CONNECTION_TIMEOUT_REQUEST)) * 1000)
											.setSocketTimeout(Integer.valueOf(basicConfig.getBasicConfiguration(
													PROP_CONFIG_CLIENT_HTTP_CONNECTION_TIMEOUT_SOCKET, 
													DEFAULT_CONFIG_CLIENT_HTTP_CONNECTION_TIMEOUT_SOCKET)) * 1000)
											.build();		
		httpClientBuilder.setDefaultRequestConfig(requestConfig);
				
		//set  pool connection if requested
		if (basicConfig.getBasicMOAIDConfigurationBoolean(
				PROP_CONFIG_CLIENT_HTTP_CONNECTION_POOL_USE, 
				true)) {
			final PoolingHttpClientConnectionManager pool = new PoolingHttpClientConnectionManager();
			pool.setDefaultMaxPerRoute(Integer.valueOf(basicConfig.getBasicConfiguration(
					PROP_CONFIG_CLIENT_HTTP_CONNECTION_POOL_MAXPERROUTE, 
					DEFAULT_CONFIG_CLIENT_HTTP_CONNECTION_POOL_MAXPERROUTE)));			
			pool.setMaxTotal(Integer.valueOf(basicConfig.getBasicConfiguration(
					PROP_CONFIG_CLIENT_HTTP_CONNECTION_POOL_MAXTOTAL, 
					DEFAULT_CONFIG_CLIENT_HTTP_CONNECTION_POOL_MAXTOTAL)));
			
			
			
			httpClientBuilder.setConnectionManager(pool);
			log.debug("Initalize http-client pool with, maxTotal: {} maxPerRoute: {}", pool.getMaxTotal(), pool.getDefaultMaxPerRoute());
					
		}
		
		try {
			log.trace("Initializing SSL Context ... ");
			final SSLContext sslContext = SSLContext.getDefault();
			HostnameVerifier hostnameVerifier = null;
			if (basicConfig.getBasicMOAIDConfigurationBoolean(
					PROP_CONFIG_CLIENT_HTTP_SSL_HOSTNAMEVERIFIER_TRUSTALL, 
					false)) {
				hostnameVerifier = new NoopHostnameVerifier();
				log.warn("HTTP client-builder deactivates SSL Host-name verification!");
			}
			
			final LayeredConnectionSocketFactory sslSocketFactory = new SSLConnectionSocketFactory(sslContext , hostnameVerifier);
			httpClientBuilder.setSSLSocketFactory(sslSocketFactory );
			
		} catch (final NoSuchAlgorithmException e) {
			log.warn("HTTP client-builder can NOT initialze SSL-Context", e);
			
		}

		
		log.info("HTTP client-builder successfuly initialized");
		
	}
	
	
}
