/*******************************************************************************
 * Copyright 2017 Graz University of Technology
 * EAAF-Core Components has been developed in a cooperation between EGIZ,  
 * A-SIT Plus, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *  
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
package at.gv.egiz.eaaf.core.impl.utils;


import at.gv.egiz.eaaf.core.api.IRequest;

/**
 * @author tlenz
 *
 */
public class TransactionIDUtils {

	  //MDC variables for logging
	  public static final String MDC_TRANSACTION_ID = "transactionId";
	  public static final String MDC_SESSION_ID = "sessionId";
	  public static final String MDC_SERVICEPROVIDER_ID = "oaId";
	
	/**
	 * Set all MDC variables from pending request to this threat context<br>
	 * These includes SessionID, TransactionID, and unique service-provider identifier
	 * 
	 * @param pendingRequest
	 */
	public static void setAllLoggingVariables(IRequest pendingRequest) {
		setTransactionId(pendingRequest.getUniqueTransactionIdentifier());
		setSessionId(pendingRequest.getUniqueSessionIdentifier());
		setServiceProviderId(pendingRequest.getServiceProviderConfiguration().getUniqueIdentifier());
		
	}
	
	/**
	 * Remove all MDC variables from this threat context
	 * 
	 */
	public static void removeAllLoggingVariables() {
		removeSessionId();
		removeTransactionId();
		removeServiceProviderId();
		
	}
	
	
	public static void setServiceProviderId(String oaUniqueId) {		
		org.slf4j.MDC.put(MDC_SERVICEPROVIDER_ID, oaUniqueId);
		
	}
	
	public static void removeServiceProviderId() {
		org.slf4j.MDC.remove(MDC_SERVICEPROVIDER_ID);
		
	}
	
	public static void setTransactionId(String pendingRequestID) {	   
		org.slf4j.MDC.put(MDC_TRANSACTION_ID, 
				"TID-" + pendingRequestID);
				    
	}
		
	public static void removeTransactionId() {
		org.slf4j.MDC.remove(MDC_TRANSACTION_ID);
		
	}
		
	public static void setSessionId(String uniqueSessionId) {	  	
		org.slf4j.MDC.put(MDC_SESSION_ID, 
				"SID-" + uniqueSessionId);
				    
	}
		
	public static void removeSessionId() {
		org.slf4j.MDC.remove(MDC_SESSION_ID);
		
	}
	
	
}
