/*******************************************************************************
 * Copyright 2017 Graz University of Technology
 * EAAF-Core Components has been developed in a cooperation between EGIZ,  
 * A-SIT Plus, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *  
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
package at.gv.egiz.eaaf.core.impl.idp.process.spring.test;


import java.io.Serializable;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import at.gv.egiz.eaaf.core.api.idp.process.ExpressionEvaluationContext;

/**
 * Adapter class for {@link ExpressionEvaluationContext}. Intended to be used for testing purposes.
 * 
 * @author tknall
 * 
 */
public class ExpressionContextAdapter implements ExpressionEvaluationContext {
 
	private static final long serialVersionUID = 1L;

	private Map<String, Serializable> ctxData = Collections.synchronizedMap(new HashMap<String, Serializable>());

	/**
	 * Returns a certain {@link Serializable} object associated with a certain {@code key}.
	 * 
	 * @param key
	 *            The key.
	 * @return The object or {@code null} if no object was found stored with that key or if a {@code null} value was
	 *         stored.
	 */
	Serializable get(String key) {
		return ctxData.get(key);
	}

	/**
	 * Stores a {@link Serializable} with a certain {@code key}.
	 * 
	 * @param key
	 *            The key.
	 * @param object
	 *            The object.
	 */
	void put(String key, Serializable object) {
		ctxData.put(key, object);
	}

	@Override
	public Map<String, Serializable> getCtx() {
		return Collections.unmodifiableMap(ctxData);
	}

}
