/*******************************************************************************
 * Copyright 2017 Graz University of Technology
 * EAAF-Core Components has been developed in a cooperation between EGIZ,  
 * A-SIT Plus, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *  
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
package at.gv.egiz.eaaf.core.impl.idp.process.spring.test;

import static at.gv.egiz.eaaf.core.impl.idp.process.ProcessInstanceState.NOT_STARTED;
import static at.gv.egiz.eaaf.core.impl.idp.process.ProcessInstanceState.SUSPENDED;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import java.io.IOException;
import java.io.InputStream;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import at.gv.egiz.eaaf.core.api.idp.process.ExecutionContext;
import at.gv.egiz.eaaf.core.api.idp.process.ProcessEngine;
import at.gv.egiz.eaaf.core.exceptions.ProcessExecutionException;
import at.gv.egiz.eaaf.core.impl.idp.module.test.TestRequestImpl;
import at.gv.egiz.eaaf.core.impl.idp.process.ProcessDefinitionParserException;
import at.gv.egiz.eaaf.core.impl.idp.process.ProcessEngineImpl;
import at.gv.egiz.eaaf.core.impl.idp.process.ProcessInstance;
import at.gv.egiz.eaaf.core.impl.idp.process.spring.SpringExpressionEvaluator;

/**
 * Tests the process engine using processes based on Spring EL referencing the process context and further Spring beans.
 * 
 * @author tknall
 * 
 */
@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration("/process/spring/test/SpringExpressionAwareProcessEngineTest-context.xml")
public class SpringExpressionAwareProcessEngineTest {

	@Autowired private static ProcessEngine pe;
	@Autowired private ApplicationContext applicationContext;
	
	private boolean isInitialized = false; 

	@Before
	public void init() throws IOException, ProcessDefinitionParserException {

		if (!isInitialized) {

			if (pe == null) {
				pe = applicationContext.getBean("processEngine", ProcessEngine.class);
				
			}

			((ProcessEngineImpl) pe).setTransitionConditionExpressionEvaluator(new SpringExpressionEvaluator());
			try (InputStream in = SpringExpressionAwareProcessEngineTest.class.getResourceAsStream("/process/spring/test/SampleProcessDefinitionWithExpression1.xml")) {
				((ProcessEngineImpl) pe).registerProcessDefinition(in);
			}
			try (InputStream in = SpringExpressionAwareProcessEngineTest.class.getResourceAsStream("/process/spring/test/SampleProcessDefinitionForSAML1Authentication.xml")) {
				((ProcessEngineImpl) pe).registerProcessDefinition(in);
			}

		//initHibernateForTesting();
		}
	}

	private static void initHibernateForTesting() throws IOException{

//		InputStream in = SpringExpressionAwareProcessEngineTest.class.getResourceAsStream("/at/gv/egovernment/moa/id/process/hibernate.configuration.test.properties");
//		Properties props = new Properties();
//		props.load(in);
//
//		try {
//			//ConfigurationDBUtils.initHibernate(props);
//			Configuration config = new Configuration();
//			config.addProperties(props);
//			//config.addAnnotatedClass(ProcessInstanceStore.class);
//			config.addAnnotatedClass(AssertionStore.class);
//			//MOASessionDBUtils.initHibernate(config, props);
//		} catch (Exception e) {
//			e.printStackTrace();
//		}
	}


	@Test
	public void testSampleProcessDefinitionWithExpression1() throws IOException, ProcessDefinitionParserException, ProcessExecutionException {

		TestRequestImpl req =  new TestRequestImpl();
		
		String piId = pe.createProcessInstance("SampleProcessWithExpression1");
		ProcessInstance pi = pe.getProcessInstance(piId);
		assertEquals(NOT_STARTED, pi.getState());
		
		
		// start process
		req.setProcessInstanceID(piId);
		pe.start(req);
		
		//processInstance should be removed when it ends
		try {
			pi = pe.getProcessInstance(piId);
			throw new ProcessExecutionException("ProcessInstance should be removed already, but it was found.");
			//assertEquals(ENDED, pi.getState());
			
		} catch (IllegalArgumentException e) {
			// do nothing because processInstance should be already removed 
			
		}
	}

	@Test
	public void testSampleProcessDefinitionForSAML1Authentication() throws IOException, ProcessDefinitionParserException, ProcessExecutionException {

		TestRequestImpl req =  new TestRequestImpl();
		
		String piId = pe.createProcessInstance("SampleProcessDefinitionForSAML1Authentication");
		ProcessInstance pi = pe.getProcessInstance(piId);
		assertEquals(NOT_STARTED, pi.getState());

		// start process
		req.setProcessInstanceID(piId);
		pe.start(req);		
		pi = pe.getProcessInstance(piId);
		assertEquals(SUSPENDED, pi.getState());
		
		ExecutionContext ec = pi.getExecutionContext();
		assertNotNull(ec);
		System.out.println(ec.keySet());

		assertNotNull(ec.get("bkuURL"));
		assertNotNull(ec.get("IdentityLink"));
		assertNotNull(ec.get("isIdentityLinkValidated"));
		assertNotNull(ec.get("SignedAuthBlock"));
		assertNotNull(ec.get("isSignedAuthBlockValidated"));
		assertNotNull(ec.get("SAML1Assertion"));
		
		pe.signal(req);
		try {
			pi = pe.getProcessInstance(piId);
			throw new ProcessExecutionException("ProcessInstance should be removed already, but it was found.");
			//assertEquals(ENDED, pi.getState());
			
		} catch (IllegalArgumentException e) {
			// do nothing because processInstance should be already removed 
			
		}



	}

}
