/*******************************************************************************
 * Copyright 2017 Graz University of Technology
 * EAAF-Core Components has been developed in a cooperation between EGIZ,  
 * A-SIT Plus, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *  
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
package at.gv.egiz.eaaf.core.impl.idp.process.spring.test.task;

import java.io.IOException;
import java.io.InputStream;
import java.util.Objects;

import org.apache.commons.io.IOUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import at.gv.egiz.eaaf.core.api.IRequest;
import at.gv.egiz.eaaf.core.api.idp.process.ExecutionContext;
import at.gv.egiz.eaaf.core.api.idp.process.Task;
import at.gv.egiz.eaaf.core.exceptions.TaskExecutionException;

/**
 * A dummy task simulating the retrieval of an IdentityLink.
 * <p/>
 * Asynchonous
 * <p>
 * Requires context data:
 * <ul>
 * <li>{@code bkuURL}</li>
 * </ul>
 * </p>
 * <p>
 * Enriches context data with:
 * <ul>
 * <li>{@code IdentityLink}</li>
 * </ul>
 * </p>
 * 
 * @author tknall
 * 
 */
@Service("GetIdentityLinkTask")
public class GetIdentityLinkTask implements Task { 

	private Logger log = LoggerFactory.getLogger(getClass());

	@Override
	public IRequest execute(IRequest penReq, ExecutionContext executionContext) throws TaskExecutionException {
		Objects.requireNonNull(executionContext.get("bkuURL"));

		log.debug("Using bkuURL in order to retrieve IdentityLink.");

		try (InputStream in = getClass().getResourceAsStream("/process/spring/test/task/IdentityLink_Max_Mustermann.xml")) {
			executionContext.put("IdentityLink", IOUtils.toString(in, "UTF-8"));
			
		} catch (IOException e) {
			throw new TaskExecutionException(null, "", e);
		}
		
		return null;
	}

}
