/*******************************************************************************
 * Copyright 2017 Graz University of Technology
 * EAAF-Core Components has been developed in a cooperation between EGIZ,  
 * A-SIT Plus, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *  
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
package at.gv.egiz.eaaf.core.api;

public interface IStatusMessenger {

	//internal error codes defined in EAAFCore
	public static final String CODES_INTERNAL_ERROR_GENERIC = "internal.00";	
	public static final String CODES_INTERNAL_ERROR_AUTH_NOSPCONFIG = "auth.00";
	public static final String CODES_INTERNAL_ERROR_AUTH_NOPENDIGREQID = "auth.26";
	public static final String CODES_INTERNAL_ERROR_AUTH_TIMEOUT = "auth.28";
	public static final String CODES_INTERNAL_ERROR_AUTH_USERSTOP = "auth.21";
	public static final String CODES_INTERNAL_ERROR_AUTH_REQUEST_INVALID = "auth.38";
	
	public static final String CODES_INTERNAL_ILLEGAL_STATE = "process.03";
	
	//external error codes defined in EAAFCore
	public static final String CODES_EXTERNAL_ERROR_GENERIC = "9199";
	public static final String CODES_EXTERNAL_ERROR_PROCESSENGINE = "1099";
	
	
	/**
	   * Get the message corresponding to a given message ID.
	   *
	   * @param messageId The ID of the message.
	   * @param parameters The parameters to fill in into the message arguments.
	   * @return The formatted message. 
	   */
	public String getMessage(String messageId, Object[] parameters);
	
	
	/**
	 * Get external errorCode from from Exception 
	 * 
	 * @param throwable
	 * @return
	 */
	public String getResponseErrorCode(Throwable throwable);
	
	
	/**
	 * Map internal to external errorCode
	 * 
	 * @param intErrorCode
	 * @return
	 */
	public String mapInternalErrorToExternalError(String intErrorCode);
	
}
