/*******************************************************************************
 * Copyright 2017 Graz University of Technology
 * EAAF-Core Components has been developed in a cooperation between EGIZ,  
 * A-SIT Plus, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *  
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
package at.gv.egiz.eaaf.core.api.gui;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import at.gv.egiz.eaaf.core.exceptions.GUIBuildException;

/**
 * @author tlenz
 *
 */
public interface IGUIFormBuilder {

	
	
	/**
	 * Parse a GUI template, with parameters into a http servlet-response 
	 * and use the default http-response content-type.
	 * <br><br>
	 *  The parser use the <code>VelocityEngine</code> as internal template evaluator. 
	 * 
     * @param httpReq http-request object
	 * @param httpResp http-response object
     * @param config Configuration object
	 * @param loggerName String, which should be used from logger
	 * 
	 * @throws GUIBuildException
	 */
	public void build(HttpServletRequest httpReq, HttpServletResponse httpResp, IGUIBuilderConfiguration config, String loggerName) throws GUIBuildException;

	/**
	 * Parse a GUI template, with parameters into a http servlet-response.
	 * <br><br>
	 *  The parser use the <code>VelocityEngine</code> as internal template evaluator. 
	 * 
	 * @param httpReq http-request object
	 * @param httpResp http-response object
	 * @param config Configuration object
	 * @param contentType http-response content-type, which should be set
	 * @param loggerName String, which should be used from logger
	 * 
	 * @throws GUIBuildException
	 */
	void build(HttpServletRequest httpReq, HttpServletResponse httpResp, IGUIBuilderConfiguration config, String contentType,
			String loggerName) throws GUIBuildException;
	

}
