/*******************************************************************************
 * Copyright 2017 Graz University of Technology
 * EAAF-Core Components has been developed in a cooperation between EGIZ,  
 * A-SIT Plus, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *  
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
package at.gv.egiz.eaaf.core.api.idp.auth.data;

import java.io.IOException;
import java.security.PublicKey;

import javax.xml.transform.TransformerException;

import org.w3c.dom.Element;

/**
 * @author tlenz
 *
 */
public interface IIdentityLink {

	/**
	   * Returns the dateOfBirth.
	   * @return Calendar
	   */
	String getDateOfBirth();

	/**
	   * Returns the familyName.
	   * @return String
	   */
	String getFamilyName();

	/**
	   * Returns the givenName.
	   * @return String
	   */
	String getGivenName();

	/**
	 * Return  the name as 'givenName + " " + familyName'<br>
	 * This method should be used any more. Use getFamilyName() and getGivenName() separately.
	 * 
	 * @return The name.
	*/
	@Deprecated
	String getName();

	/**
	   * Returns the identificationValue.
		 * <code>"identificationValue"</code> is the translation of <code>"Stammzahl"</code>.
	   * @return String
	   */
	String getIdentificationValue();

	/**
	 * Returns the identificationType.
	 * <code>"identificationType"</code> type of the identificationValue in the IdentityLink.
	 * @return String
	 */
	String getIdentificationType();

	/**
	   * Sets the dateOfBirth.
	   * @param dateOfBirth The dateOfBirth to set
	   */
	void setDateOfBirth(String dateOfBirth);

	/**
	   * Sets the familyName.
	   * @param familyName The familyName to set
	   */
	void setFamilyName(String familyName);

	/**
	   * Sets the givenName.
	   * @param givenName The givenName to set
	   */
	void setGivenName(String givenName);

	/**
	   * Sets the identificationValue.
		 * <code>"identificationValue"</code> is the translation of <code>"Stammzahl"</code>.
	   * @param identificationValue The identificationValue to set
	   */
	void setIdentificationValue(String identificationValue);

	/**
	 * Sets the Type of the identificationValue.
	 * @param identificationType The type of identificationValue to set
	 */
	void setIdentificationType(String identificationType);

	/**
	   * Returns the samlAssertion.
	   * @return Element
	   */
	Element getSamlAssertion();

	/**
	   * Returns the samlAssertion.
	   * @return Element
	   */
	String getSerializedSamlAssertion();

	/**
	   * Sets the samlAssertion and the serializedSamlAssertion.
	   * @param samlAssertion The samlAssertion to set
	   */
	void setSamlAssertion(Element samlAssertion) throws TransformerException, IOException;

	/**
	   * Returns the dsigReferenceTransforms.
	   * @return Element[]
	   */
	Element[] getDsigReferenceTransforms();

	/**
	   * Sets the dsigReferenceTransforms.
	   * @param dsigReferenceTransforms The dsigReferenceTransforms to set
	   */
	void setDsigReferenceTransforms(Element[] dsigReferenceTransforms);

	/**
	   * Returns the publicKey.
	   * @return PublicKey[]
	   */
	PublicKey[] getPublicKey();

	/**
	   * Sets the publicKey.
	   * @param publicKey The publicKey to set
	   */
	void setPublicKey(PublicKey[] publicKey);

	/**
	   * Returns the prPerson.
	   * @return Element
	   */
	Element getPrPerson();

	/**
	   * Sets the prPerson.
	   * @param prPerson The prPerson to set
	   */
	void setPrPerson(Element prPerson);

	/**
	   * Returns the issuing time of the identity link SAML assertion.
	   *
	   * @return The issuing time of the identity link SAML assertion.
	   */
	String getIssueInstant();

	/**
	   * Sets the issuing time of the identity link SAML assertion.
	   *
	   * @param issueInstant The issueInstant to set.
	   */
	void setIssueInstant(String issueInstant);

}