/*******************************************************************************
 * Copyright 2017 Graz University of Technology
 * EAAF-Core Components has been developed in a cooperation between EGIZ,  
 * A-SIT Plus, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *  
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/


package at.gv.egiz.eaaf.core.exceptions;

import java.io.PrintStream;
import java.io.PrintWriter;

/**
 * An exception occurred evaluating an XPath.
 * 
 */
public class XPathException extends RuntimeException {
  /**
	 * 
	 */
	private static final long serialVersionUID = 1736311265333034392L;
/** The wrapped exception. */
  private Throwable wrapped;
  
  /**
   * Create a <code>XPathException</code>.
   * 
   * @param message The exception message.
   * @param wrapped The exception being the likely cause of this exception.
   */
  public XPathException(String message, Throwable wrapped) {
    super(message);
    this.wrapped = wrapped; 
  }
  
  public XPathException(String string) {
	  super(string);
}

/**
   * Return the wrapped exception.
   * 
   * @return The wrapped exception being the likely cause of this exception.
   */
  public Throwable getWrapped() {
    return wrapped;
  }

  /**
   * @see java.lang.Throwable#printStackTrace(java.io.PrintStream)
   */
  public void printStackTrace(PrintStream s) {
    super.printStackTrace(s);
    if (getWrapped() != null) {
      s.print("Caused by: ");
      getWrapped().printStackTrace(s);
    }
  }

  /**
   * @see java.lang.Throwable#printStackTrace(java.io.PrintWriter)
   */
  public void printStackTrace(PrintWriter s) {
    super.printStackTrace(s);
    if (getWrapped() != null) {
      s.print("Caused by: ");
      getWrapped().printStackTrace(s);
    }
  }
 
}
