package at.gv.egiz.eaaf.modules.auth.sl20.utils;

import java.security.cert.X509Certificate;

import com.fasterxml.jackson.databind.JsonNode;

import at.gv.egiz.eaaf.modules.auth.sl20.data.VerificationResult;
import at.gv.egiz.eaaf.modules.auth.sl20.exceptions.SL20Exception;
import at.gv.egiz.eaaf.modules.auth.sl20.exceptions.SLCommandoBuildException;
import at.gv.egiz.eaaf.modules.auth.sl20.exceptions.SLCommandoParserException;

public interface IJOSETools {

	/**
	 * Create a JWS signature
	 * 
	 * @param payLoad Payload to sign
	 * @throws SLCommandoBuildException 
	 */
	public String createSignature(String payLoad) throws SLCommandoBuildException;

	/**
	 * Validates a JWS signature
	 * 
	 * @param serializedContent
	 * @return
	 * @throws SLCommandoParserException
	 * @throws SL20Exception 
	 */
	public VerificationResult validateSignature(String serializedContent) throws SL20Exception;
	
	/**
	 * Get the encryption certificate for SL2.0 End-to-End encryption
	 * 
	 * @return
	 */
	public X509Certificate getEncryptionCertificate();

	/**
	 * Decrypt a serialized JWE token
	 * 
	 * @param compactSerialization Serialized JWE token
	 * @return decrypted payload
	 * @throws SL20Exception 
	 */
	public JsonNode decryptPayload(String compactSerialization) throws SL20Exception;
	 
}
