package at.gv.egiz.eid.authhandler.modules.sigverify.moasig.api.data;

import iaik.x509.X509Certificate;
import java.util.Date;

import org.springframework.lang.Nullable;

import at.gv.egiz.eid.authhandler.modules.sigverify.moasig.exceptions.MOASigServiceException;


public interface IGenericSignatureVerificationResponse {

	/**
	 * Returns the signing time
	 * 
	 * @return Signing time, or null if signature contains no time information
	 */
	@Nullable
	Date getSigningDateTime();
	
	/**
	   * Returns the signatureCheckCode.
	   * @return int
	   */
	int getSignatureCheckCode();
	
	/**
	   * Returns the certificateCheckCode.
	   * @return int
	   */
	int getCertificateCheckCode();

	/**
	   * Returns the qualifiedCertificate.
	   * @return boolean
	   */
	boolean isQualifiedCertificate();

	/**
	   * Returns the X509 certificate.
	   * @return X509Certificate, or null if no certificate information exists
	   * @throws MOASigServiceException if X509 certificate can not be deserialized
	   */
	@Nullable
	X509Certificate getX509Certificate() throws MOASigServiceException;

	
	/**
	 * Returns the X509 certificate in serialized form
	 * 
	 * @return Serialized X509 certificate, or null if no certificate information exists
	 */
	@Nullable
	byte[] getX509CertificateEncoded();
	
	/**
	   * Returns the publicAuthority.
	   * @return boolean
	   */
	boolean isPublicAuthority();
	
	/**
	   * Returns the publicAuthorityCode.
	   * @return String OID, or null if no OID exists
	   */
	@Nullable
	String getPublicAuthorityCode();

}
