package at.gv.egiz.eid.authhandler.modules.sigverify.moasig.impl.data;

import java.io.Serializable;
import java.security.cert.CertificateException;
import iaik.x509.X509Certificate;
import java.util.Date;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import at.gv.egiz.eid.authhandler.modules.sigverify.moasig.api.data.IGenericSignatureVerificationResponse;
import at.gv.egiz.eid.authhandler.modules.sigverify.moasig.exceptions.MOASigServiceException;
import at.gv.egiz.eid.authhandler.modules.sigverify.moasig.exceptions.MOASigServiceParserException;

public class GenericSignatureVerificationResponse implements IGenericSignatureVerificationResponse, Serializable {

	private static final long serialVersionUID = -7751001050689401118L;
	private static final Logger log = LoggerFactory.getLogger(GenericSignatureVerificationResponse.class);
	
	
	/** The signing time */
	private Date signingDateTime;
	
	/** The signatureCheckCode to be stored */
	private int signatureCheckCode;
	
	/** The certificateCheckCode to be stored */
	private int certificateCheckCode;
		
	/** The publicAuthority to be stored */
	private boolean publicAuthority;
  
	/** The publicAuthorityCode to be stored */
	private String publicAuthorityCode;
  
	/** The qualifiedCertificate to be stored */
	private boolean qualifiedCertificate;
  	
	private byte[] x509CertificateEncoded;
	
	@Override
	public Date getSigningDateTime() {
		return this.signingDateTime;
		
	}

	@Override
	public int getSignatureCheckCode() {
		return this.signatureCheckCode;
		
	}

	@Override
	public int getCertificateCheckCode() {
		return this.certificateCheckCode;
		
	}

	@Override
	public boolean isQualifiedCertificate() {
		return this.qualifiedCertificate;
		
	}

	@Override
	public X509Certificate getX509Certificate() throws MOASigServiceException {
		if (x509CertificateEncoded != null) {
			try {
				return new X509Certificate(x509CertificateEncoded);
				
			} catch (CertificateException e) {
				log.error("Can NOT parse X509 certifcate in " + GenericSignatureVerificationResponse.class.getName(), e);
				throw new MOASigServiceParserException("service.moasig.01", null, e);
			}
			
		}
		
		return null;
		
	}

	@Override
	public byte[] getX509CertificateEncoded() {
		return this.getX509CertificateEncoded();
		
	}
	
	@Override
	public boolean isPublicAuthority() {
		return this.publicAuthority;
		
	}

	@Override
	public String getPublicAuthorityCode() {
		return this.publicAuthorityCode;
		
	}

	public void setSigningDateTime(Date signingDateTime) {
		this.signingDateTime = signingDateTime;
	}

	public void setSignatureCheckCode(int signatureCheckCode) {
		this.signatureCheckCode = signatureCheckCode;
	}

	public void setCertificateCheckCode(int certificateCheckCode) {
		this.certificateCheckCode = certificateCheckCode;
	}

	public void setPublicAuthority(boolean publicAuthority) {
		this.publicAuthority = publicAuthority;
	}

	public void setPublicAuthorityCode(String publicAuthorityCode) {
		this.publicAuthorityCode = publicAuthorityCode;
	}

	public void setQualifiedCertificate(boolean qualifiedCertificate) {
		this.qualifiedCertificate = qualifiedCertificate;
	}

	public void setX509CertificateEncoded(byte[] x509CertificateEncoded) {
		this.x509CertificateEncoded = x509CertificateEncoded;
	}
	
	

}
