/*******************************************************************************
 * Copyright 2017 Graz University of Technology
 * EAAF-Core Components has been developed in a cooperation between EGIZ,  
 * A-SIT Plus, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *  
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
package at.gv.egiz.eaaf.modules.pvp2.api.binding;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.opensaml.saml2.core.RequestAbstractType;
import org.opensaml.saml2.core.StatusResponseType;
import org.opensaml.ws.message.encoder.MessageEncodingException;
import org.opensaml.xml.security.SecurityException;
import org.opensaml.xml.security.credential.Credential;

import at.gv.egiz.eaaf.core.api.IRequest;
import at.gv.egiz.eaaf.modules.pvp2.exception.PVP2Exception;

public interface IEncoder {
	
	/**
	 * 
	 * @param req The http request
	 * @param resp The http response
	 * @param request The SAML2 request object
	 * @param targetLocation URL, where the request should be transmit
	 * @param relayState token for session handling
	 * @param credentials Credential to sign the request object
	 * @param pendingReq Internal MOA-ID request object that contains session-state informations but never null
	 * @throws MessageEncodingException
	 * @throws SecurityException
	 * @throws PVP2Exception
	 */
	public void encodeRequest(HttpServletRequest req, 
			HttpServletResponse resp, RequestAbstractType request, String targetLocation, String relayState, Credential credentials, IRequest pendingReq) 
					throws MessageEncodingException, SecurityException, PVP2Exception;
	
	/**
	 * Encoder SAML Response
	 * @param req The http request
	 * @param resp The http response
	 * @param response The SAML2 repsonse object
	 * @param targetLocation URL, where the request should be transmit
	 * @param relayState token for session handling
	 * @param credentials Credential to sign the response object
	 * @param pendingReq Internal MOA-ID request object that contains session-state informations but never null
	 * @throws MessageEncodingException
	 * @throws SecurityException
	 */
	public void encodeRespone(HttpServletRequest req, 
			HttpServletResponse resp, StatusResponseType response, String targetLocation, String relayState, Credential credentials, IRequest pendingReq) 
					throws MessageEncodingException, SecurityException, PVP2Exception;
}
