/*******************************************************************************
 * Copyright 2017 Graz University of Technology
 * EAAF-Core Components has been developed in a cooperation between EGIZ,  
 * A-SIT Plus, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *  
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
package at.gv.egiz.eaaf.modules.pvp2.impl.builder.reqattr;

import java.util.Map.Entry;

import javax.xml.namespace.QName;

import org.opensaml.Configuration;
import org.opensaml.common.impl.AbstractSAMLObjectMarshaller;
import org.opensaml.xml.XMLObject;
import org.opensaml.xml.io.MarshallingException;
import org.opensaml.xml.util.XMLHelper;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;

import at.gv.egiz.eaaf.modules.pvp2.api.reqattr.EAAFRequestedAttribute;

public class EAAFRequestedAttributeMarshaller extends AbstractSAMLObjectMarshaller {
    protected final void marshallAttributes(final XMLObject samlElement,
    	    final Element domElement) throws MarshallingException {
    	final EAAFRequestedAttribute requestedAttr = (EAAFRequestedAttribute) samlElement;

    	if (requestedAttr.getName() != null) {
    	    domElement.setAttributeNS(null, EAAFRequestedAttribute.NAME_ATTRIB_NAME, requestedAttr.getName());
    	}

    	if (requestedAttr.getNameFormat() != null) {
    	    domElement.setAttributeNS(null, EAAFRequestedAttribute.NAME_FORMAT_ATTR, requestedAttr.getNameFormat());
    	}

    	if (requestedAttr.getFriendlyName() != null) {
    	    domElement.setAttributeNS(null, EAAFRequestedAttribute.FRIENDLY_NAME_ATT, requestedAttr.getFriendlyName());
    	}

    	if (requestedAttr.getIsRequiredXSBoolean() != null) {
    	    domElement.setAttributeNS(null, EAAFRequestedAttribute.IS_REQUIRED_ATTR, requestedAttr.getIsRequiredXSBoolean());
    	}

    	Attr attr;
    	for (Entry<QName, String> entry : requestedAttr.getUnknownAttributes()
    		.entrySet()) {
    	    attr = XMLHelper.constructAttribute(domElement.getOwnerDocument(),
    		    entry.getKey());
    	    attr.setValue(entry.getValue());
    	    domElement.setAttributeNodeNS(attr);
    	    if (Configuration.isIDAttribute(entry.getKey())
    		    || requestedAttr.getUnknownAttributes().isIDAttribute(
    			    entry.getKey())) {
    		attr.getOwnerElement().setIdAttributeNode(attr, true);
    	    }
    	}
        }
}
