/*******************************************************************************
 * Copyright 2017 Graz University of Technology
 * EAAF-Core Components has been developed in a cooperation between EGIZ,  
 * A-SIT Plus, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *  
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
package at.gv.egiz.eaaf.modules.pvp2.impl.builder.reqattr;

import javax.xml.namespace.QName;

import org.opensaml.common.impl.AbstractSAMLObjectUnmarshaller;
import org.opensaml.xml.XMLObject;
import org.opensaml.xml.io.UnmarshallingException;
import org.opensaml.xml.util.XMLHelper;
import org.w3c.dom.Attr;

import at.gv.egiz.eaaf.modules.pvp2.PVPConstants;
import at.gv.egiz.eaaf.modules.pvp2.api.reqattr.EAAFRequestedAttribute;

public class EAAFRequestedAttributeUnmarshaller extends AbstractSAMLObjectUnmarshaller {
    protected final void processChildElement(final XMLObject parentSAMLObject, final XMLObject childSAMLObject) throws UnmarshallingException {
    	final EAAFRequestedAttribute requestedAttr = (EAAFRequestedAttribute) parentSAMLObject;
    	final QName childQName = childSAMLObject.getElementQName();
    
    	if ("AttributeValue".equals(childQName.getLocalPart())
    		&& childQName.getNamespaceURI().equals(PVPConstants.EIDAT10_SAML_NS)) {
    	    requestedAttr.getAttributeValues().add(childSAMLObject);
    	
    	} else
    	    super.processChildElement(parentSAMLObject, childSAMLObject);

    }

    protected final void processAttribute(final XMLObject samlObject, final Attr attribute) throws UnmarshallingException {
    	final EAAFRequestedAttribute requestedAttr = (EAAFRequestedAttribute) samlObject;
    	if (attribute.getLocalName().equals(EAAFRequestedAttribute.NAME_ATTRIB_NAME)) {
    	    requestedAttr.setName(attribute.getValue());
    	
    	} else if (attribute.getLocalName().equals(EAAFRequestedAttribute.NAME_FORMAT_ATTR)) {
    	    requestedAttr.setNameFormat(attribute.getValue());
    	    
    	} else if (attribute.getLocalName().equals(EAAFRequestedAttribute.FRIENDLY_NAME_ATT)) {
    	    requestedAttr.setFriendlyName(attribute.getValue());
    	    
    	} else if (attribute.getLocalName().equals(EAAFRequestedAttribute.IS_REQUIRED_ATTR)) {
    	    requestedAttr.setIsRequired(attribute.getValue());

    	} else {
    	    final QName attribQName = XMLHelper.getNodeQName(attribute);
    	    if (attribute.isId()) {
    	    	requestedAttr.getUnknownAttributes().registerID(attribQName);
    	    }
    	    requestedAttr.getUnknownAttributes().put(attribQName,
    		    attribute.getValue());
    	}
   }
}
