/*******************************************************************************
 * Copyright 2017 Graz University of Technology
 * EAAF-Core Components has been developed in a cooperation between EGIZ,  
 * A-SIT Plus, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *  
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
package at.gv.egiz.eaaf.modules.pvp2.impl.message;

import java.io.Serializable;

import org.opensaml.saml2.metadata.EntityDescriptor;
import org.opensaml.saml2.metadata.provider.MetadataProviderException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Element;

import at.gv.egiz.eaaf.modules.pvp2.api.message.InboundMessageInterface;
import at.gv.egiz.eaaf.modules.pvp2.api.metadata.IPVPMetadataProvider;
import at.gv.egiz.eaaf.modules.pvp2.exception.NoMetadataInformationException;
 
/**
 * @author tlenz
 *
 */
public class InboundMessage implements InboundMessageInterface, Serializable{
	private static final Logger log = LoggerFactory.getLogger(InboundMessage.class);
	
	private static final long serialVersionUID = 2395131650841669663L;
	
	private Element samlMessage = null;
	private boolean verified = false;
	private String entityID = null;
	private String relayState = null;
	
	
	public EntityDescriptor getEntityMetadata(IPVPMetadataProvider  metadataProvider) throws NoMetadataInformationException {
		try {
			if (metadataProvider == null)
				throw new NullPointerException("No PVP MetadataProvider found.");
			
			return metadataProvider.getEntityDescriptor(this.entityID);
			
		} catch (MetadataProviderException e) {
			log.warn("No Metadata for EntitiyID " + entityID); 
			throw new NoMetadataInformationException();
		}			
	}
	
	/**
	 * @param entitiyID the entitiyID to set
	 */
	public void setEntityID(String entitiyID) {
		this.entityID = entitiyID;
	}
	
	public void setVerified(boolean verified) {
		this.verified = verified;
	}
		
	/**
	 * @param relayState the relayState to set
	 */
	public void setRelayState(String relayState) {
		this.relayState = relayState;
	}
	
	public void setSAMLMessage(Element msg) {
		this.samlMessage = msg;
	}
	
	/* (non-Javadoc)
	 * @see at.gv.egovernment.moa.id.protocols.pvp2x.messages.PVP21InboundMessage#getRelayState()
	 */
	@Override
	public String getRelayState() {
		return relayState;
	}

	/* (non-Javadoc)
	 * @see at.gv.egovernment.moa.id.protocols.pvp2x.messages.PVP21InboundMessage#getEntityID()
	 */
	@Override
	public String getEntityID() {
		return entityID;
	}

	/* (non-Javadoc)
	 * @see at.gv.egovernment.moa.id.protocols.pvp2x.messages.PVP21InboundMessage#isVerified()
	 */
	@Override
	public boolean isVerified() {
		return verified;
	}

	/* (non-Javadoc)
	 * @see at.gv.egovernment.moa.id.protocols.pvp2x.messages.PVP21InboundMessage#getInboundMessage()
	 */
	@Override
	public Element getInboundMessage() {
		return samlMessage;
	}

}
