/*******************************************************************************
 * Copyright 2017 Graz University of Technology
 * EAAF-Core Components has been developed in a cooperation between EGIZ,  
 * A-SIT Plus, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *  
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
package at.gv.egiz.eaaf.modules.pvp2.impl.metadata;

import java.util.ArrayList;
import java.util.List;

import org.opensaml.saml2.metadata.provider.FilterException;
import org.opensaml.saml2.metadata.provider.MetadataFilter;
import org.opensaml.xml.XMLObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * @author tlenz
 *
 */
public class MetadataFilterChain implements MetadataFilter {
	private static final Logger log = LoggerFactory.getLogger(MetadataFilterChain.class);
		
	
	private List<MetadataFilter> filters = new ArrayList<MetadataFilter>();
		
	/**
	 * Return all actually used Metadata filters
	 * 
	 * @return List of Metadata filters
	 */
	public List<MetadataFilter> getFilters() {
		return filters;
	}
	
	/**
	 * Add a new Metadata filter to filterchain
	 * 
	 * @param filter 
	 */
	public void addFilter(MetadataFilter filter) {
		filters.add(filter);
	}
	
	
	/* (non-Javadoc)
	 * @see org.opensaml.saml2.metadata.provider.MetadataFilter#doFilter(org.opensaml.xml.XMLObject)
	 */
	@Override
	public void doFilter(XMLObject arg0) throws FilterException {
		for (MetadataFilter filter : filters) {
			log.trace("Use EAAFMetadataFilter " + filter.getClass().getName());
			filter.doFilter(arg0);
		}

	}

}
