/*******************************************************************************
 * Copyright 2017 Graz University of Technology
 * EAAF-Core Components has been developed in a cooperation between EGIZ,  
 * A-SIT Plus, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *  
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
package at.gv.egiz.eaaf.modules.pvp2.impl.opensaml;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.Writer;

import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;
import org.opensaml.common.binding.SAMLMessageContext;
import org.opensaml.saml2.binding.encoding.HTTPPostEncoder;
import org.opensaml.ws.message.encoder.MessageEncodingException;
import org.opensaml.ws.transport.http.HTTPOutTransport;
import org.opensaml.ws.transport.http.HTTPTransportUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import at.gv.egiz.eaaf.core.api.gui.IVelocityGUIBuilderConfiguration;
import at.gv.egiz.eaaf.core.api.gui.IVelocityGuiFormBuilder;

/**
 * @author tlenz
 *
 */
public class HTTPPostEncoderWithOwnTemplate extends HTTPPostEncoder {
	private static final Logger log = LoggerFactory.getLogger(HTTPPostEncoderWithOwnTemplate.class);
	
	
	private final VelocityEngine velocityEngine;
	private final IVelocityGUIBuilderConfiguration guiConfig;
	private final IVelocityGuiFormBuilder guiBuilder;
	
	/**
	 * @param engine
	 * @param templateId
	 */
	public HTTPPostEncoderWithOwnTemplate(IVelocityGUIBuilderConfiguration guiConfig, IVelocityGuiFormBuilder guiBuilder, VelocityEngine engine) {
		super(engine, null);
		this.velocityEngine = engine;
		this.guiConfig = guiConfig;
		this.guiBuilder = guiBuilder;
		
	}

    /**
     * Base64 and POST encodes the outbound message and writes it to the outbound transport.
     * 
     * @param messageContext current message context
     * @param endpointURL endpoint URL to which to encode message
     * 
     * @throws MessageEncodingException thrown if there is a problem encoding the message
     */
    @Override
	protected void postEncode(SAMLMessageContext messageContext, String endpointURL) throws MessageEncodingException {
    	log.debug("Invoking Velocity template to create POST body");
        InputStream is = null;
        try {        	
        	//build Velocity Context from GUI input paramters
			final VelocityContext context = guiBuilder.generateVelocityContextFromConfiguration(guiConfig);
        	
			//load template
			is = guiBuilder.getTemplateInputStream(guiConfig);
						
			//populate velocity context with SAML2 parameters
            populateVelocityContext(context, messageContext, endpointURL);

            //populate transport parameter
            final HTTPOutTransport outTransport = (HTTPOutTransport) messageContext.getOutboundMessageTransport();
            HTTPTransportUtils.addNoCacheHeaders(outTransport);
            HTTPTransportUtils.setUTF8Encoding(outTransport);
            HTTPTransportUtils.setContentType(outTransport, "text/html");

            //evaluate template and write content to response
            final Writer out = new OutputStreamWriter(outTransport.getOutgoingStream(), "UTF-8");                        
            velocityEngine.evaluate(context, out, "SAML2_POST_BINDING", new BufferedReader(new InputStreamReader(is)));            
            out.flush();
            
        } catch (final Exception e) {
        	log.error("Error invoking Velocity template", e);
            throw new MessageEncodingException("Error creating output document", e);
            
        } finally {
			if (is != null) {
				try {
					is.close();
					
				} catch (final IOException e) {
					log.error("Can NOT close GUI-Template InputStream.", e);
				}
			}
        	
		}
    }
}
