/*******************************************************************************
 * Copyright 2017 Graz University of Technology
 * EAAF-Core Components has been developed in a cooperation between EGIZ,  
 * A-SIT Plus, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *  
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
package at.gv.egiz.eaaf.modules.pvp2.impl.opensaml;

import org.opensaml.common.binding.SAMLMessageContext;
import org.opensaml.saml2.binding.encoding.HTTPRedirectDeflateEncoder;
import org.opensaml.ws.message.MessageContext;
import org.opensaml.ws.message.encoder.MessageEncodingException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import at.gv.egiz.eaaf.modules.pvp2.impl.opensaml.initialize.EAAFDefaultSAML2Bootstrap;

/**
 * @author tlenz
 *
 */
public class StringRedirectDeflateEncoder extends HTTPRedirectDeflateEncoder {
	private static final Logger log = LoggerFactory.getLogger(StringRedirectDeflateEncoder.class);
	
	private String redirectURL = null;
	
	public void encode(MessageContext messageContext)
			throws MessageEncodingException {
		if (!(messageContext instanceof SAMLMessageContext)) {
			log.error("Invalid message context type, this encoder only support SAMLMessageContext");
			throw new MessageEncodingException(
					"Invalid message context type, this encoder only support SAMLMessageContext");
		}

		//load default PVP security configurations
		EAAFDefaultSAML2Bootstrap.initializeDefaultPVPConfiguration();
		
		SAMLMessageContext samlMsgCtx = (SAMLMessageContext) messageContext;

		String endpointURL = getEndpointURL(samlMsgCtx).buildURL();

		setResponseDestination(samlMsgCtx.getOutboundSAMLMessage(), endpointURL);

		removeSignature(samlMsgCtx);

		String encodedMessage = deflateAndBase64Encode(samlMsgCtx
				.getOutboundSAMLMessage());

		redirectURL = buildRedirectURL(samlMsgCtx, endpointURL,
				encodedMessage);
	}

	/**
	 * @return the redirectURL
	 */
	public String getRedirectURL() {
		return redirectURL;
	}
	
	
}
