/*******************************************************************************
 * Copyright 2017 Graz University of Technology
 * EAAF-Core Components has been developed in a cooperation between EGIZ,  
 * A-SIT Plus, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *  
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
package at.gv.egiz.eaaf.modules.pvp2.impl.validation.metadata;

import javax.xml.transform.dom.DOMSource;
import javax.xml.validation.Schema;
import javax.xml.validation.Validator;

import org.opensaml.common.xml.SAMLSchemaBuilder;
import org.opensaml.saml2.metadata.provider.FilterException;
import org.opensaml.saml2.metadata.provider.MetadataFilter;
import org.opensaml.xml.XMLObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.xml.sax.SAXException;

import at.gv.egiz.eaaf.modules.pvp2.exception.SchemaValidationException;

/**
 * @author tlenz
 *
 */
public class SchemaValidationFilter implements MetadataFilter {
	private static final Logger log = LoggerFactory.getLogger(SchemaValidationFilter.class);
	private boolean isActive = true;
	
	public SchemaValidationFilter() {
	}
	
	/**
	 * 
	 */
	public SchemaValidationFilter(boolean useSchemaValidation) {
		this.isActive = useSchemaValidation;
	}
	
	
	/* (non-Javadoc)
	 * @see org.opensaml.saml2.metadata.provider.MetadataFilter#doFilter(org.opensaml.xml.XMLObject)
	 */
	@Override
	public void doFilter(XMLObject arg0) throws FilterException {
		
		String errString = null;
		
		if (isActive) {
			try {
				Schema test = SAMLSchemaBuilder.getSAML11Schema();
				Validator val = test.newValidator();
				DOMSource source = new DOMSource(arg0.getDOM());		
				val.validate(source);
				log.info("Metadata Schema validation check done OK");
				return;
			
			} catch (SAXException e) {
				if (log.isDebugEnabled() || log.isTraceEnabled())
					log.warn("Metadata Schema validation FAILED with exception:", e);
				else
					log.warn("Metadata Schema validation FAILED with message: "+ e.getMessage());

				errString = e.getMessage();
				
			} catch (Exception e) {
				if (log.isDebugEnabled() || log.isTraceEnabled())
					log.warn("Metadata Schema validation FAILED with exception:", e);
				else
					log.warn("Metadata Schema validation FAILED with message: "+ e.getMessage());
				
				errString = e.getMessage();
				
			}
			
			throw new FilterException(
					new SchemaValidationException("pvp2.26", 
							new Object[] {"Metadata Schema validation FAILED with message: " + errString}));
						
		} else		
			log.info("Metadata Schema validation check is DEACTIVATED!");
		
	}

}
